"""
This type stub file was generated by pyright.
"""

import numpy as np
import scipy.sparse as ss
from enum import Enum
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Type, Union
from .compat import Client, dask_Array, pd_DataFrame, pd_Series
from .sklearn import LGBMClassifier, LGBMRanker, LGBMRegressor

"""Distributed training with LightGBM and dask.distributed.

This module enables you to perform distributed training with LightGBM on
dask.Array and dask.DataFrame collections.

It is based on dask-lightgbm, which was based on dask-xgboost.
"""
_DaskCollection = ...
_DaskMatrixLike = ...
_DaskVectorLike = ...
_DaskPart = Union[np.ndarray, pd_DataFrame, pd_Series, ss.spmatrix]
_PredictionDtype = Union[Type[np.float32], Type[np.float64], Type[np.int32], Type[np.int64]]
_HostWorkers = ...
class _DatasetNames(Enum):
    """Placeholder names used by lightgbm.dask internals to say 'also evaluate the training data'.

    Avoid duplicating the training data when the validation set refers to elements of training data.
    """
    TRAINSET = ...
    SAMPLE_WEIGHT = ...
    INIT_SCORE = ...
    GROUP = ...


class _DaskLGBMModel:
    @property
    def client_(self) -> Client:
        """:obj:`dask.distributed.Client`: Dask client.

        This property can be passed in the constructor or updated
        with ``model.set_params(client=client)``.
        """
        ...
    


class DaskLGBMClassifier(LGBMClassifier, _DaskLGBMModel):
    """Distributed version of lightgbm.LGBMClassifier."""
    def __init__(self, boosting_type: str = ..., num_leaves: int = ..., max_depth: int = ..., learning_rate: float = ..., n_estimators: int = ..., subsample_for_bin: int = ..., objective: Optional[Union[Callable, str]] = ..., class_weight: Optional[Union[dict, str]] = ..., min_split_gain: float = ..., min_child_weight: float = ..., min_child_samples: int = ..., subsample: float = ..., subsample_freq: int = ..., colsample_bytree: float = ..., reg_alpha: float = ..., reg_lambda: float = ..., random_state: Optional[Union[int, np.random.RandomState]] = ..., n_jobs: int = ..., silent: bool = ..., importance_type: str = ..., client: Optional[Client] = ..., **kwargs: Any) -> None:
        """Docstring is inherited from the lightgbm.LGBMClassifier.__init__."""
        ...
    
    _base_doc = ...

    def __getstate__(self) -> Dict[Any, Any]:
        ...
    
    def fit(self, X: _DaskMatrixLike, y: _DaskCollection, sample_weight: Optional[_DaskVectorLike] = ..., init_score: Optional[_DaskCollection] = ..., eval_set: Optional[List[Tuple[_DaskMatrixLike, _DaskCollection]]] = ..., eval_names: Optional[List[str]] = ..., eval_sample_weight: Optional[List[_DaskVectorLike]] = ..., eval_class_weight: Optional[List[Union[dict, str]]] = ..., eval_init_score: Optional[List[_DaskCollection]] = ..., eval_metric: Optional[Union[Callable, str, List[Union[Callable, str]]]] = ..., early_stopping_rounds: Optional[int] = ..., **kwargs: Any) -> DaskLGBMClassifier:
        """Docstring is inherited from the lightgbm.LGBMClassifier.fit."""
        ...
    
    _base_doc = ...

    def predict(self, X: _DaskMatrixLike, **kwargs: Any) -> dask_Array:
        """Docstring is inherited from the lightgbm.LGBMClassifier.predict."""
        ...
    
    def predict_proba(self, X: _DaskMatrixLike, **kwargs: Any) -> dask_Array:
        """Docstring is inherited from the lightgbm.LGBMClassifier.predict_proba."""
        ...
    
    def to_local(self) -> LGBMClassifier:
        """Create regular version of lightgbm.LGBMClassifier from the distributed version.

        Returns
        -------
        model : lightgbm.LGBMClassifier
            Local underlying model.
        """
        ...
    


class DaskLGBMRegressor(LGBMRegressor, _DaskLGBMModel):
    """Distributed version of lightgbm.LGBMRegressor."""
    def __init__(self, boosting_type: str = ..., num_leaves: int = ..., max_depth: int = ..., learning_rate: float = ..., n_estimators: int = ..., subsample_for_bin: int = ..., objective: Optional[Union[Callable, str]] = ..., class_weight: Optional[Union[dict, str]] = ..., min_split_gain: float = ..., min_child_weight: float = ..., min_child_samples: int = ..., subsample: float = ..., subsample_freq: int = ..., colsample_bytree: float = ..., reg_alpha: float = ..., reg_lambda: float = ..., random_state: Optional[Union[int, np.random.RandomState]] = ..., n_jobs: int = ..., silent: bool = ..., importance_type: str = ..., client: Optional[Client] = ..., **kwargs: Any) -> None:
        """Docstring is inherited from the lightgbm.LGBMRegressor.__init__."""
        ...
    
    _base_doc = ...
    
    def __getstate__(self) -> Dict[Any, Any]:
        ...
    
    def fit(self, X: _DaskMatrixLike, y: _DaskCollection, sample_weight: Optional[_DaskVectorLike] = ..., init_score: Optional[_DaskVectorLike] = ..., eval_set: Optional[List[Tuple[_DaskMatrixLike, _DaskCollection]]] = ..., eval_names: Optional[List[str]] = ..., eval_sample_weight: Optional[List[_DaskVectorLike]] = ..., eval_init_score: Optional[List[_DaskVectorLike]] = ..., eval_metric: Optional[Union[Callable, str, List[Union[Callable, str]]]] = ..., early_stopping_rounds: Optional[int] = ..., **kwargs: Any) -> DaskLGBMRegressor:
        """Docstring is inherited from the lightgbm.LGBMRegressor.fit."""
        ...

    def predict(self, X: _DaskMatrixLike, **kwargs) -> dask_Array:
        """Docstring is inherited from the lightgbm.LGBMRegressor.predict."""
        ...
    
    def to_local(self) -> LGBMRegressor:
        """Create regular version of lightgbm.LGBMRegressor from the distributed version.

        Returns
        -------
        model : lightgbm.LGBMRegressor
            Local underlying model.
        """
        ...
    


class DaskLGBMRanker(LGBMRanker, _DaskLGBMModel):
    """Distributed version of lightgbm.LGBMRanker."""
    def __init__(self, boosting_type: str = ..., num_leaves: int = ..., max_depth: int = ..., learning_rate: float = ..., n_estimators: int = ..., subsample_for_bin: int = ..., objective: Optional[Union[Callable, str]] = ..., class_weight: Optional[Union[dict, str]] = ..., min_split_gain: float = ..., min_child_weight: float = ..., min_child_samples: int = ..., subsample: float = ..., subsample_freq: int = ..., colsample_bytree: float = ..., reg_alpha: float = ..., reg_lambda: float = ..., random_state: Optional[Union[int, np.random.RandomState]] = ..., n_jobs: int = ..., silent: bool = ..., importance_type: str = ..., client: Optional[Client] = ..., **kwargs: Any) -> None:
        """Docstring is inherited from the lightgbm.LGBMRanker.__init__."""
        ...
    
    _base_doc = ...
    
    def __getstate__(self) -> Dict[Any, Any]:
        ...
    
    def fit(self, X: _DaskMatrixLike, y: _DaskCollection, sample_weight: Optional[_DaskVectorLike] = ..., init_score: Optional[_DaskVectorLike] = ..., group: Optional[_DaskVectorLike] = ..., eval_set: Optional[List[Tuple[_DaskMatrixLike, _DaskCollection]]] = ..., eval_names: Optional[List[str]] = ..., eval_sample_weight: Optional[List[_DaskVectorLike]] = ..., eval_init_score: Optional[List[_DaskVectorLike]] = ..., eval_group: Optional[List[_DaskVectorLike]] = ..., eval_metric: Optional[Union[Callable, str, List[Union[Callable, str]]]] = ..., eval_at: Iterable[int] = ..., early_stopping_rounds: Optional[int] = ..., **kwargs: Any) -> DaskLGBMRanker:
        """Docstring is inherited from the lightgbm.LGBMRanker.fit."""
        ...
    
    def predict(self, X: _DaskMatrixLike, **kwargs: Any) -> dask_Array:
        """Docstring is inherited from the lightgbm.LGBMRanker.predict."""
        ...
    
    def to_local(self) -> LGBMRanker:
        """Create regular version of lightgbm.LGBMRanker from the distributed version.

        Returns
        -------
        model : lightgbm.LGBMRanker
            Local underlying model.
        """
        ...
    





from datetime import date, datetime
from decimal import Decimal
from itertools import chain
from typing import (
    Any,
    Callable,
    Dict,
    Iterator,
    List,
    Optional,
    Set,
    Tuple,
    Type,
    Union,
)
from uuid import UUID

from django.db.models.base import Model
from django.db.models.expressions import BaseExpression, Expression
from django.db.models.sql.query import Query, RawQuery

FORCE: Any

class SQLCompiler:
    query: Any = ...
    connection: Any = ...
    using: Any = ...
    quote_cache: Any = ...
    select: Any = ...
    annotation_col_map: Any = ...
    klass_info: Any = ...
    ordering_parts: Any = ...
    def __init__(
        self, query: Union[Query, RawQuery], connection: Any, using: Optional[str]
    ) -> None: ...
    col_count: Any = ...
    def setup_query(self) -> None: ...
    has_extra_select: Any = ...
    def pre_sql_setup(
        self,
    ) -> Tuple[
        List[Tuple[Expression, Tuple[str, Union[List[Any], Tuple[str, str]]], None]],
        List[Tuple[Expression, Tuple[str, List[Union[int, str]], bool]]],
        List[Tuple[str, List[float]]],
    ]: ...
    def get_group_by(
        self,
        select: List[Tuple[BaseExpression, Tuple[str, List[float]], Optional[str]]],
        order_by: List[Tuple[Expression, Tuple[str, List[Union[int, str]], bool]]],
    ) -> List[Tuple[str, List[float]]]: ...
    def collapse_group_by(
        self,
        expressions: List[Expression],
        having: Union[List[Expression], Tuple[Any, ...]],
    ) -> List[Expression]: ...
    def get_select(
        self,
    ) -> Tuple[
        List[Tuple[Expression, Tuple[str, List[Union[int, str]]], Optional[str]]],
        Optional[Dict[str, Any]],
        Dict[str, int],
    ]: ...
    def get_order_by(self) -> List[Tuple[Expression, Tuple[str, List[Any], bool]]]: ...
    def get_extra_select(
        self,
        order_by: List[Tuple[Expression, Tuple[str, List[Any], bool]]],
        select: List[Tuple[Expression, Tuple[str, List[float]], Optional[str]]],
    ) -> List[Tuple[Expression, Tuple[str, List[Any]], None]]: ...
    def quote_name_unless_alias(self, name: str) -> str: ...
    def compile(
        self, node: Any, select_format: Any = ...
    ) -> Tuple[str, Union[List[Optional[int]], Tuple[int, int]]]: ...
    def get_combinator_sql(
        self, combinator: str, all: bool
    ) -> Tuple[List[str], Union[List[int], List[str]]]: ...
    def as_sql(self, with_limits: bool = ..., with_col_aliases: bool = ...) -> Any: ...
    def get_default_columns(
        self,
        start_alias: Optional[str] = ...,
        opts: Optional[Any] = ...,
        from_parent: Optional[Type[Model]] = ...,
    ) -> List[Expression]: ...
    def get_distinct(self) -> Tuple[List[Any], List[Any]]: ...
    def find_ordering_name(
        self,
        name: str,
        opts: Any,
        alias: Optional[str] = ...,
        default_order: str = ...,
        already_seen: Optional[
            Set[Tuple[Optional[Tuple[Tuple[str, str]]], Tuple[Tuple[str, str]]]]
        ] = ...,
    ) -> List[Tuple[Expression, bool]]: ...
    def get_from_clause(self) -> Tuple[List[str], List[Union[int, str]]]: ...
    def get_related_selections(
        self,
        select: List[Tuple[Expression, Optional[str]]],
        opts: Optional[Any] = ...,
        root_alias: Optional[str] = ...,
        cur_depth: int = ...,
        requested: Optional[
            Union[Dict[str, Dict[str, Dict[str, Dict[Any, Any]]]], bool]
        ] = ...,
        restricted: Optional[bool] = ...,
    ) -> List[Dict[str, Any]]: ...
    def get_select_for_update_of_arguments(self) -> Any: ...
    def deferred_to_columns(self) -> Dict[Type[Model], Set[str]]: ...
    def get_converters(
        self, expressions: List[Expression]
    ) -> Dict[int, Tuple[List[Callable[..., Any]], Expression]]: ...
    def apply_converters(
        self,
        rows: chain[Any],
        converters: Dict[int, Tuple[List[Callable[..., Any]], Expression]],
    ) -> Iterator[
        Union[
            List[Optional[Union[bytes, datetime, int, str]]],
            List[Optional[Union[date, Decimal, float, str]]],
            List[Optional[Union[datetime, float, str, UUID]]],
        ]
    ]: ...
    def results_iter(
        self,
        results: Optional[
            Union[Iterator[Any], List[List[Tuple[Union[int, str]]]]]
        ] = ...,
        tuple_expected: bool = ...,
        chunked_fetch: bool = ...,
        chunk_size: int = ...,
    ) -> Iterator[Any]: ...
    def has_results(self) -> bool: ...
    def execute_sql(
        self, result_type: str = ..., chunked_fetch: bool = ..., chunk_size: int = ...
    ) -> Optional[Any]: ...
    def as_subquery_condition(
        self, alias: str, columns: List[str], compiler: SQLCompiler
    ) -> Tuple[str, Tuple[Any, ...]]: ...
    def explain_query(self) -> Iterator[str]: ...

class SQLInsertCompiler(SQLCompiler):
    returning_fields: Any = ...
    returning_params: Any = ...
    def field_as_sql(self, field: Any, val: Any) -> Any: ...
    def prepare_value(self, field: Any, value: Any) -> Any: ...
    def pre_save_val(self, field: Any, obj: Any) -> Any: ...
    def assemble_as_sql(self, fields: Any, value_rows: Any) -> Any: ...
    def as_sql(self) -> Any: ...  # type: ignore [override]

class SQLDeleteCompiler(SQLCompiler):
    def single_alias(self) -> Any: ...
    def as_sql(self) -> Any: ...  # type: ignore [override]

class SQLUpdateCompiler(SQLCompiler):
    def as_sql(self) -> Any: ...  # type: ignore [override]

class SQLAggregateCompiler(SQLCompiler):
    col_count: Any = ...
    def as_sql(self) -> Any: ...  # type: ignore [override]

def cursor_iter(
    cursor: Any, sentinel: Any, col_count: Optional[int], itersize: int
) -> Iterator[Any]: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

class Key(object):
    def __init__(self, *names): ...
    @property
    def name(self): ...
    def __hash__(self): ...
    def __repr__(self): ...
    def __eq__(self, other): ...

SHIFT = ...
CONTROL = ...
ALT = ...
META = ...  # That Mac thingy

UP = ...
DOWN = ...
LEFT = ...
RIGHT = ...
PAGEUP = ...
PAGEDOWN = ...

INSERT = ...
DELETE = ...
HOME = ...
END = ...

ESCAPE = ...
BACKSPACE = ...

F1 = ...
F2 = ...
F3 = ...
F4 = ...
F5 = ...
F6 = ...
F7 = ...
F8 = ...
F9 = ...
F10 = ...
F11 = ...
F12 = ...

SPACE = ...
ENTER = ...
TAB = ...



from ._export import (
    export_graphviz as export_graphviz,
    plot_tree as plot_tree,
    export_text as export_text,
)
from ._classes import (
    BaseDecisionTree as BaseDecisionTree,
    DecisionTreeClassifier as DecisionTreeClassifier,
    DecisionTreeRegressor as DecisionTreeRegressor,
    ExtraTreeClassifier as ExtraTreeClassifier,
    ExtraTreeRegressor as ExtraTreeRegressor,
)

__all__ = [
    "BaseDecisionTree",
    "DecisionTreeClassifier",
    "DecisionTreeRegressor",
    "ExtraTreeClassifier",
    "ExtraTreeRegressor",
    "export_graphviz",
    "plot_tree",
    "export_text",
]



"""Parameter models for this project."""

from pathlib import Path
from typing import Any, get_origin

from pydantic import BaseModel, validator
from ruamel.yaml import YAML

YAML_INDENT = 2
yaml = YAML()
yaml.indent(mapping=YAML_INDENT, sequence=YAML_INDENT, offset=YAML_INDENT)
yaml.preserve_quotes = True


class YamlModel(BaseModel):
    """Model of a YAML file with automatic schema generation.

    Updates a JSON schema next to the YAML file with each initialization.
    """

    def __init__(self, data_file: Path):
        """Initialize and update the schema."""
        params = self.get_params(data_file)
        self.update_schema(data_file)
        super().__init__(**params)

    def get_params(self, data_file: Path) -> dict[str, Any]:
        """Get parameters from file."""
        yaml = YAML()
        yaml.indent(YAML_INDENT)
        return (
            yaml.load(data_file)
            if data_file.exists() and data_file.read_text(encoding="utf-8")
            else {}
        )

    def update_schema(self, data_file: Path):
        schema_file = data_file.with_name(f"{data_file.stem}_schema.json")
        schema_file.write_text(
            encoding="utf-8", data=f"{self.schema_json(indent=YAML_INDENT)}\n"
        )


class SynchronizedPathsYamlModel(YamlModel):
    """Model of a YAML file that synchronizes paths back to the file.

    For example, synchronize complex path structures back to `params.yaml` DVC files for
    pipeline orchestration.
    """

    def __init__(self, data_file: Path):
        """Initialize, update the schema, and synchronize paths in the file."""
        super().__init__(data_file)

    def get_params(self, data_file: Path) -> dict[str, Any]:
        """Get parameters from file, synchronizing paths in the file."""
        yaml = YAML()
        yaml.indent(YAML_INDENT)
        params = (
            yaml.load(data_file)
            if data_file.exists() and data_file.read_text(encoding="utf-8")
            else {}
        )
        params |= self.get_paths()
        yaml.dump(params, data_file)
        return params

    def get_paths(self) -> dict[str, dict[str, str]]:
        """Get all paths specified in paths-type models."""
        maybe_excludes = self.__exclude_fields__
        excludes = set(maybe_excludes.keys()) if maybe_excludes else set()
        defaults: dict[str, dict[str, str]] = {}
        for key, field in self.__fields__.items():
            if generic_ := get_origin(field.type_):
                type_ = type(generic_)
            else:
                type_ = field.type_
            if key not in excludes and issubclass(type_, DefaultPathsModel):
                defaults[key] = type_.get_paths()
        return defaults


class DefaultPathsModel(BaseModel):
    """All fields must be path-like and have defaults specified in this model."""

    class Config:
        @staticmethod
        def schema_extra(schema: dict[str, Any], model):
            """Replace backslashes with forward slashes in paths."""
            if schema.get("required"):
                raise TypeError(
                    f"Defaults must be specified in {model}, derived from {DefaultPathsModel}."
                )
            for (field, prop), type_ in zip(
                schema["properties"].items(),
                (field.type_ for field in model.__fields__.values()),
                strict=True,
            ):
                if not issubclass(type_, Path):
                    raise TypeError(
                        f"Field <{field}> is not Path-like in {model}, derived from {DefaultPathsModel}."
                    )
                default = prop.get("default")
                if isinstance(default, list | tuple):
                    default = [item.replace("\\", "/") for item in default]
                elif isinstance(default, dict):
                    default = {
                        key: value.replace("\\", "/") for key, value in default.items()
                    }
                else:
                    default = default.replace("\\", "/")
                prop["default"] = default

    @validator("*", always=True, pre=True, each_item=True)
    def check_pathlike(cls, value, field):
        """Check that the value is path-like."""
        if not issubclass(field.type_, Path):
            raise TypeError(
                f"Field is not Path-like in {cls}, derived from {DefaultPathsModel}."
            )
        return value

    @classmethod
    def get_paths(cls) -> dict[str, Any]:
        """Get the paths for this model."""
        return {
            key: value["default"] for key, value in cls.schema()["properties"].items()
        }


class CreatePathsModel(DefaultPathsModel):
    """Parent directories will be created for all fields in this model."""

    @validator("*", always=True, pre=True, each_item=True)
    def create_directories(cls, value):
        """Create directories associated with each value."""
        path = Path(value)
        if path.is_file():
            return value
        directory = path.parent if path.suffix else path
        directory.mkdir(parents=True, exist_ok=True)
        return value



from ..gloo.texture import Texture2D

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from .wrappers import read_pixels

def _screenshot(viewport=None, alpha=True): ...

KEYWORDS = ...

def check_variable(name): ...
def check_identifier(name): ...
def check_enum(enum, name=None, valid=None): ...

vert_draw: str = ...

frag_draw: str = ...

def draw_texture(tex: Texture2D): ...



from typing import Any, Callable, Literal, TypeVar
from numpy import ndarray
from ._base import NeighborsBase, KNeighborsMixin, RadiusNeighborsMixin
from .._typing import Int, Float, MatrixLike

NearestNeighbors_Self = TypeVar("NearestNeighbors_Self", bound="NearestNeighbors")


class NearestNeighbors(KNeighborsMixin, RadiusNeighborsMixin, NeighborsBase):
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str = ...

    def __init__(
        self,
        *,
        n_neighbors: Int = 5,
        radius: Float = 1.0,
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        metric: str | Callable = "minkowski",
        p: Float = 2,
        metric_params: None | dict = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: NearestNeighbors_Self, X: MatrixLike, y: Any = None
    ) -> NearestNeighbors_Self:
        ...



PY37 = ...
PY38 = ...
PYPY = ...

def set_function_name(f, name, cls): ...
def is_platform_little_endian() -> bool: ...
def is_platform_windows() -> bool: ...
def is_platform_linux() -> bool: ...
def is_platform_mac() -> bool: ...
def is_platform_32bit() -> bool: ...



from typing import Any, ClassVar, Iterable, Iterator, Mapping, TypeVar
from ..utils import check_array as check_array
from operator import itemgetter as itemgetter
from array import array as array
from numpy import ndarray
from numbers import Number as Number
from ..base import BaseEstimator, TransformerMixin
from scipy.sparse import spmatrix
from .._typing import MatrixLike, ArrayLike
from collections.abc import Mapping, Iterable

DictVectorizer_Self = TypeVar("DictVectorizer_Self", bound="DictVectorizer")

# Authors: Lars Buitinck
#          Dan Blanchard <dblanchard@ets.org>
# License: BSD 3 clause


import numpy as np
import scipy.sparse as sp


class DictVectorizer(TransformerMixin, BaseEstimator):
    feature_names_: list = ...
    vocabulary_: dict = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, *, dtype=..., separator: str = "=", sparse: bool = True, sort: bool = True
    ) -> None:
        ...

    def fit(
        self: DictVectorizer_Self, X: Mapping | Iterable[Mapping], y: Any = None
    ) -> DictVectorizer_Self:
        ...

    def fit_transform(
        self,
        X: Iterator[Any] | Mapping | Iterable[Mapping] | list[dict[str, int]],
        y: Any = None,
    ) -> ndarray | spmatrix:
        ...

    def inverse_transform(
        self, X: MatrixLike | ArrayLike, dict_type=...
    ) -> list[Mapping]:
        ...

    def transform(
        self, X: ArrayLike | Mapping[str, ArrayLike] | Iterator[Mapping[str, ArrayLike]]
    ) -> ndarray | spmatrix:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def restrict(
        self: DictVectorizer_Self, support: ArrayLike, indices: bool = False
    ) -> DictVectorizer_Self:
        ...



from typing import Any, Literal, Sequence
from ..utils import (
    check_pandas_support as check_pandas_support,
    get_chunk_n_rows as get_chunk_n_rows,
)
from collections import OrderedDict as OrderedDict
from numpy import ndarray
from pandas import DataFrame, Series
from ..externals._arff import ArffSparseDataType as ArffSparseDataType
from scipy.sparse import spmatrix
from gzip import GzipFile
from collections.abc import Generator as Generator
import itertools
import re

import numpy as np
import scipy as sp


def load_arff_from_gzip_file(
    gzip_file: GzipFile,
    parser: Literal["pandas", "liac-arff"],
    output_type: Literal["numpy", "sparse", "pandas"],
    openml_columns_info: dict,
    feature_names_to_select: list[str] | Sequence[str],
    target_names_to_select: Sequence[str] | list[Any | str],
    shape: None | tuple[int, int] = None,
) -> tuple[
    DataFrame | ndarray | spmatrix,
    DataFrame | ndarray | Series,
    DataFrame | None,
    list[str] | None,
]:
    ...



import datetime
from typing import Any, Dict, Optional, Sequence, Tuple

from django.db import models
from django.http import HttpRequest, HttpResponse
from django.views.generic.base import View
from django.views.generic.detail import (
    BaseDetailView,
    SingleObjectTemplateResponseMixin,
)
from django.views.generic.list import (
    MultipleObjectMixin,
    MultipleObjectTemplateResponseMixin,
)

class YearMixin:
    year_format: str = ...
    year: Optional[str] = ...
    def get_year_format(self) -> str: ...
    def get_year(self) -> str: ...
    def get_next_year(self, date: datetime.date) -> Optional[datetime.date]: ...
    def get_previous_year(self, date: datetime.date) -> Optional[datetime.date]: ...

class MonthMixin:
    month_format: str = ...
    month: Optional[str] = ...
    def get_month_format(self) -> str: ...
    def get_month(self) -> str: ...
    def get_next_month(self, date: datetime.date) -> Optional[datetime.date]: ...
    def get_previous_month(self, date: datetime.date) -> Optional[datetime.date]: ...

class DayMixin:
    day_format: str = ...
    day: Optional[str] = ...
    def get_day_format(self) -> str: ...
    def get_day(self) -> str: ...
    def get_next_day(self, date: datetime.date) -> Optional[datetime.date]: ...
    def get_previous_day(self, date: datetime.date) -> Optional[datetime.date]: ...

class WeekMixin:
    week_format: str = ...
    week: Optional[str] = ...
    def get_week_format(self) -> str: ...
    def get_week(self) -> str: ...
    def get_next_week(self, date: datetime.date) -> Optional[datetime.date]: ...
    def get_previous_week(self, date: datetime.date) -> Optional[datetime.date]: ...

class DateMixin:
    date_field: Optional[str] = ...
    allow_future: bool = ...
    def get_date_field(self) -> str: ...
    def get_allow_future(self) -> bool: ...
    def uses_datetime_field(self) -> bool: ...

DatedItems = Tuple[Optional[Sequence[datetime.date]], Sequence[object], Dict[str, Any]]

class BaseDateListView(MultipleObjectMixin, DateMixin, View):
    date_list_period: str = ...
    def render_to_response(
        self, context: Dict[str, Any], **response_kwargs: Any
    ) -> HttpResponse: ...
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def get_dated_items(self) -> DatedItems: ...
    def get_dated_queryset(self, **lookup: Any) -> models.query.QuerySet[Any]: ...
    def get_date_list_period(self) -> str: ...
    def get_date_list(
        self,
        queryset: models.query.QuerySet[Any],
        date_type: Optional[str] = ...,
        ordering: str = ...,
    ) -> models.query.QuerySet[Any]: ...

class BaseArchiveIndexView(BaseDateListView): ...
class ArchiveIndexView(MultipleObjectTemplateResponseMixin, BaseArchiveIndexView): ...

class BaseYearArchiveView(YearMixin, BaseDateListView):
    make_object_list: bool = ...
    def get_make_object_list(self) -> bool: ...

class YearArchiveView(MultipleObjectTemplateResponseMixin, BaseYearArchiveView): ...
class BaseMonthArchiveView(YearMixin, MonthMixin, BaseDateListView): ...
class MonthArchiveView(MultipleObjectTemplateResponseMixin, BaseMonthArchiveView): ...
class BaseWeekArchiveView(YearMixin, WeekMixin, BaseDateListView): ...
class WeekArchiveView(MultipleObjectTemplateResponseMixin, BaseWeekArchiveView): ...
class BaseDayArchiveView(YearMixin, MonthMixin, DayMixin, BaseDateListView): ...
class DayArchiveView(MultipleObjectTemplateResponseMixin, BaseDayArchiveView): ...
class BaseTodayArchiveView(BaseDayArchiveView): ...
class TodayArchiveView(MultipleObjectTemplateResponseMixin, BaseTodayArchiveView): ...
class BaseDateDetailView(
    YearMixin, MonthMixin, DayMixin, DateMixin, BaseDetailView
): ...
class DateDetailView(SingleObjectTemplateResponseMixin, BaseDateDetailView): ...

def timezone_today() -> datetime.date: ...



from typing import ClassVar, Iterable, Literal, Sequence, TypeVar
from numpy.random import RandomState
from scipy import sparse as sparse
from scipy.sparse._coo import coo_matrix
from ..model_selection._split import BaseShuffleSplit
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot
from joblib import effective_n_jobs as effective_n_jobs
from scipy.sparse import spmatrix
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import (
    check_random_state as check_random_state,
    check_consistent_length as check_consistent_length,
    check_is_fitted as check_is_fitted,
    column_or_1d as column_or_1d,
)
from abc import ABC, abstractmethod
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from functools import partial as partial
from ..base import RegressorMixin, MultiOutputMixin
from ..model_selection import check_cv as check_cv
from ..utils import check_array as check_array, check_scalar as check_scalar
from ._base import LinearModel
from .._typing import MatrixLike, ArrayLike, Float, Int
from ..model_selection import BaseCrossValidator

LinearModelCV_Self = TypeVar("LinearModelCV_Self", bound="LinearModelCV")
MultiTaskElasticNetCV_Self = TypeVar(
    "MultiTaskElasticNetCV_Self", bound="MultiTaskElasticNetCV"
)
MultiTaskLassoCV_Self = TypeVar("MultiTaskLassoCV_Self", bound="MultiTaskLassoCV")
ElasticNet_Self = TypeVar("ElasticNet_Self", bound="ElasticNet")
MultiTaskElasticNet_Self = TypeVar(
    "MultiTaskElasticNet_Self", bound="MultiTaskElasticNet"
)

# Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#         Fabian Pedregosa <fabian.pedregosa@inria.fr>
#         Olivier Grisel <olivier.grisel@ensta.org>
#         Gael Varoquaux <gael.varoquaux@inria.fr>
#
# License: BSD 3 clause

import sys
import warnings
import numbers

import numpy as np


def lasso_path(
    X: MatrixLike | ArrayLike,
    y: MatrixLike | ArrayLike,
    *,
    eps: Float = 1e-3,
    n_alphas: Int = 100,
    alphas: None | ArrayLike = None,
    precompute: Literal["auto", "auto"] | MatrixLike | bool = "auto",
    Xy: None | MatrixLike | ArrayLike = None,
    copy_X: bool = True,
    coef_init: None | ArrayLike = None,
    verbose: int | bool = False,
    return_n_iter: bool = False,
    positive: bool = False,
    **params,
) -> tuple[ndarray, ndarray, ndarray] | tuple[ndarray, ndarray, ndarray, list[int]]:
    ...


def enet_path(
    X: MatrixLike | ArrayLike,
    y: MatrixLike | ArrayLike,
    *,
    l1_ratio: Float = 0.5,
    eps: Float = 1e-3,
    n_alphas: Int = 100,
    alphas: None | ArrayLike = None,
    precompute: Literal["auto", "auto"] | MatrixLike | bool = "auto",
    Xy: None | MatrixLike | ArrayLike = None,
    copy_X: bool = True,
    coef_init: None | ArrayLike = None,
    verbose: int | bool = False,
    return_n_iter: bool = False,
    positive: bool = False,
    check_input: bool = True,
    **params,
) -> tuple[ndarray, ndarray, ndarray, list[int]]:
    ...


###############################################################################
# ElasticNet model


class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    dual_gap_: float | ndarray = ...
    n_iter_: list[int] = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    path = ...

    def __init__(
        self,
        alpha: Float = 1.0,
        *,
        l1_ratio: Float = 0.5,
        fit_intercept: bool = True,
        precompute: MatrixLike | bool = False,
        max_iter: Int = 1000,
        copy_X: bool = True,
        tol: Float = 1e-4,
        warm_start: bool = False,
        positive: bool = False,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...

    def fit(
        self: ElasticNet_Self,
        X: coo_matrix | MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        check_input: bool = True,
    ) -> ElasticNet_Self:
        ...

    @property
    def sparse_coef_(self) -> spmatrix:
        ...


###############################################################################
# Lasso model


class Lasso(ElasticNet):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int | list[int] = ...
    intercept_: float | ndarray = ...
    sparse_coef_: spmatrix = ...
    dual_gap_: float | ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    path = ...

    def __init__(
        self,
        alpha: Float = 1.0,
        *,
        fit_intercept: bool = True,
        precompute: MatrixLike | bool = False,
        copy_X: bool = True,
        max_iter: Int = 1000,
        tol: Float = 1e-4,
        warm_start: bool = False,
        positive: bool = False,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...


class LinearModelCV(MultiOutputMixin, LinearModel, ABC):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        eps: float = 1e-3,
        n_alphas: int = 100,
        alphas: None | ndarray = None,
        fit_intercept: bool = True,
        precompute: str = "auto",
        max_iter: int = 1000,
        tol: float = 1e-4,
        copy_X: bool = True,
        cv: None | int = None,
        verbose: bool = False,
        n_jobs=None,
        positive: bool = False,
        random_state: None | int = None,
        selection: str = "cyclic",
    ) -> None:
        ...

    @staticmethod
    @abstractmethod
    def path(X, y, **kwargs):
        ...

    def fit(
        self: LinearModelCV_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> LinearModelCV_Self | LassoCV:
        ...


class LassoCV(RegressorMixin, LinearModelCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    dual_gap_: float | ndarray = ...
    alphas_: ndarray = ...
    mse_path_: ndarray = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...
    alpha_: float = ...

    path = ...

    def __init__(
        self,
        *,
        eps: Float = 1e-3,
        n_alphas: Int = 100,
        alphas: None | ArrayLike = None,
        fit_intercept: bool = True,
        precompute: Literal["auto", "auto"] | MatrixLike | bool = "auto",
        max_iter: Int = 1000,
        tol: Float = 1e-4,
        copy_X: bool = True,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        verbose: int | bool = False,
        n_jobs: None | Int = None,
        positive: bool = False,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...


class ElasticNetCV(RegressorMixin, LinearModelCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    dual_gap_: float = ...
    alphas_: ndarray = ...
    mse_path_: ndarray = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...
    l1_ratio_: float = ...
    alpha_: float = ...

    _parameter_constraints: ClassVar[dict] = ...

    path = ...

    def __init__(
        self,
        *,
        l1_ratio: float | Sequence[float] = 0.5,
        eps: Float = 1e-3,
        n_alphas: Int = 100,
        alphas: None | ArrayLike = None,
        fit_intercept: bool = True,
        precompute: Literal["auto", "auto"] | MatrixLike | bool = "auto",
        max_iter: Int = 1000,
        tol: Float = 1e-4,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        copy_X: bool = True,
        verbose: int | bool = 0,
        n_jobs: None | Int = None,
        positive: bool = False,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...


###############################################################################
# Multi Task ElasticNet and Lasso models (with joint feature selection)


class MultiTaskElasticNet(Lasso):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    sparse_coef_: spmatrix = ...
    eps_: float = ...
    dual_gap_: float = ...
    n_iter_: int = ...
    coef_: ndarray = ...
    intercept_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...
    for param in ("precompute", "positive"):
        pass

    def __init__(
        self,
        alpha: Float = 1.0,
        *,
        l1_ratio: Float = 0.5,
        fit_intercept: bool = True,
        copy_X: bool = True,
        max_iter: Int = 1000,
        tol: Float = 1e-4,
        warm_start: bool = False,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...

    def fit(
        self: MultiTaskElasticNet_Self, X: ArrayLike, y: MatrixLike
    ) -> MultiTaskElasticNet_Self | MultiTaskLasso:
        ...


class MultiTaskLasso(MultiTaskElasticNet):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    sparse_coef_: spmatrix = ...
    eps_: float = ...
    dual_gap_: ndarray = ...
    n_iter_: int = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        alpha: Float = 1.0,
        *,
        fit_intercept: bool = True,
        copy_X: bool = True,
        max_iter: Int = 1000,
        tol: Float = 1e-4,
        warm_start: bool = False,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...


class MultiTaskElasticNetCV(RegressorMixin, LinearModelCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    dual_gap_: float = ...
    n_iter_: int = ...
    l1_ratio_: float = ...
    alphas_: ndarray = ...
    mse_path_: ndarray = ...
    alpha_: float = ...
    coef_: ndarray = ...
    intercept_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    path = ...

    def __init__(
        self,
        *,
        l1_ratio: float | Sequence[float] = 0.5,
        eps: Float = 1e-3,
        n_alphas: Int = 100,
        alphas: None | ArrayLike = None,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: Float = 1e-4,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        copy_X: bool = True,
        verbose: int | bool = 0,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...

    # This is necessary as LinearModelCV now supports sample_weight while
    # MultiTaskElasticNet does not (yet).
    def fit(
        self: MultiTaskElasticNetCV_Self, X: ArrayLike, y: MatrixLike
    ) -> MultiTaskElasticNetCV_Self:
        ...


class MultiTaskLassoCV(RegressorMixin, LinearModelCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    dual_gap_: float = ...
    n_iter_: int = ...
    alphas_: ndarray = ...
    mse_path_: ndarray = ...
    alpha_: float = ...
    coef_: ndarray = ...
    intercept_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    path = ...

    def __init__(
        self,
        *,
        eps: Float = 1e-3,
        n_alphas: Int = 100,
        alphas: None | ArrayLike = None,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: Float = 1e-4,
        copy_X: bool = True,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        verbose: int | bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        selection: Literal["cyclic", "random", "cyclic"] = "cyclic",
    ) -> None:
        ...

    # This is necessary as LinearModelCV now supports sample_weight while
    # MultiTaskElasticNet does not (yet).
    def fit(
        self: MultiTaskLassoCV_Self, X: ArrayLike, y: MatrixLike
    ) -> MultiTaskLassoCV_Self:
        ...



from vispy.util.svg.color import Color
from vispy.color.color_array import ColorArray
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np

from vispy.color import ColorArray
from vispy.gloo import VertexBuffer
from vispy.visuals.shaders import Variable
from vispy.visuals.visual import Visual

_VERTEX_SHADER: str = ...

_FRAGMENT_SHADER: str = ...

class WindbarbVisual(Visual):

    _shaders: dict = ...

    def __init__(self, **kwargs): ...
    def set_data(
        self,
        pos: ArrayLike | None = None,
        wind: ArrayLike | None = None,
        trig: bool = True,
        size: float | ArrayLike = 50.0,
        antialias: float = 1.0,
        edge_width: None | float = 1.0,
        edge_color: ColorArray | Color | str = "black",
        face_color: ColorArray | Color | str = "white",
    ): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...
    def _compute_bounds(self, axis, view): ...



from typing import Callable, Literal

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ..shaders import Function

class BaseFilter(object):
    def _attach(self, visual): ...
    def _detach(self, visual): ...

class Filter(BaseFilter):
    def __init__(
        self,
        vcode: None | str | Callable = None,
        vhook: Literal["pre", "post"] = "post",
        vpos: int = 5,
        fcode: None | str | Callable = None,
        fhook: Literal["pre", "post"] = "post",
        fpos: int = 5,
    ): ...
    @property
    def attached(self): ...
    def _attach(self, visual): ...
    def _detach(self, visual): ...



from typing import Iterable

class _DeprecatedModule:
    deprmod = ...
    deprmodto = ...
    removals = ...
    moved = ...
    self_dir = ...
    def __init__(self, deprmod, deprmodto=..., removals=..., moved=...) -> None: ...
    def __dir__(self) -> Iterable[str]: ...
    def __getattr__(self, name: str): ...



from typing import Literal
import numpy as np
from numpy.typing import ArrayLike

class BinaryTree:

    data = ...
    sample_weight = ...
    sum_weight = ...

    idx_array = ...
    node_data = ...
    node_bounds = ...

    leaf_size = ...
    n_levels = ...
    n_nodes = ...

    dist_metric = ...
    euclidean = ...

    # variables to keep track of building & querying stats
    n_trims = ...
    n_leaves = ...
    n_splits = ...
    n_calls = ...

    valid_metrics = ...

    def __init__(
        self, data, leaf_size=40, metric="minkowski", sample_weight=None, **kwargs
    ): ...
    def get_tree_stats(self) -> tuple[int, ...]: ...
    def reset_n_calls(self) -> None: ...
    def get_n_calls(self) -> int: ...
    def get_arrays(self) -> tuple[np.ndarray, ...]: ...
    def dist(self, x1, x2, size) -> float: ...
    def rdist(self, x1, x2, size) -> float: ...
    def query(
        self,
        X: ArrayLike,
        k: int = 1,
        return_distance: bool = True,
        dualtree: bool = False,
        breadth_first: bool = False,
        sort_results: bool = True,
    ) -> np.ndarray | tuple[np.ndarray, np.ndarray]: ...
    def query_radius(
        self,
        X: ArrayLike,
        r: float | ArrayLike,
        return_distance: bool = False,
        count_only: bool = False,
        sort_results: bool = False,
    ) -> np.ndarray | tuple[np.ndarray, np.ndarray]: ...
    def kernel_density(
        self,
        X: ArrayLike,
        h: float,
        kernel: Literal[
            "gaussian", "tophat", "epanechnikov", "exponential", "linear", "cosine"
        ] = "gaussian",
        atol: float = 0,
        rtol: float = 1e-8,
        breadth_first: bool = True,
        return_log: bool = False,
    ) -> np.ndarray: ...
    def two_point_correlation(
        self, X: ArrayLike, r: ArrayLike, dualtree: bool = False
    ) -> np.ndarray: ...



from vispy.util.event import Event
from vispy.scene.widgets.viewbox import ViewBox

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
import numpy as np

from .widget import Widget
from ...visuals import AxisVisual

class AxisWidget(Widget):
    def __init__(self, orientation: str = "left", **kwargs): ...
    def on_resize(self, event: Event): ...
    def _update_axis(self): ...
    def _axis_ends(self): ...
    def link_view(self, view: ViewBox): ...
    def _view_changed(self, event=None): ...



from typing import Callable
from numpy.typing import ArrayLike
import math
import types
from itertools import permutations
from ...classes.graph import Graph

__all__ = [
    "categorical_node_match",
    "categorical_edge_match",
    "categorical_multiedge_match",
    "numerical_node_match",
    "numerical_edge_match",
    "numerical_multiedge_match",
    "generic_node_match",
    "generic_edge_match",
    "generic_multiedge_match",
]

def copyfunc(f, name=None): ...
def allclose(x, y, rtol: float = 1.0000000000000001e-05, atol: float = 1e-08): ...

categorical_doc: str = ...

def categorical_node_match(attr: str | ArrayLike, default): ...

categorical_edge_match = ...

def categorical_multiedge_match(attr: str | ArrayLike, default): ...

# Docstrings for categorical functions.
categorical_node_match.__doc__ = ...
categorical_edge_match.__doc__ = ...
tmpdoc = ...
tmpdoc = ...
categorical_multiedge_match.__doc__ = ...

numerical_doc: str = ...

def numerical_node_match(
    attr: str | ArrayLike,
    default,
    rtol: float = 1.0000000000000001e-05,
    atol: float = 1e-08,
): ...

numerical_edge_match = ...

def numerical_multiedge_match(
    attr: str | ArrayLike,
    default,
    rtol: float = 1.0000000000000001e-05,
    atol: float = 1e-08,
): ...

# Docstrings for numerical functions.
numerical_node_match.__doc__ = ...
numerical_edge_match.__doc__ = ...
tmpdoc = ...
tmpdoc = ...
numerical_multiedge_match.__doc__ = ...

generic_doc: str = ...

def generic_node_match(attr: str | ArrayLike, default, op: ArrayLike | Callable): ...

generic_edge_match = ...

def generic_multiedge_match(
    attr: str | ArrayLike, default, op: ArrayLike | Callable
): ...

# Docstrings for numerical functions.
generic_node_match.__doc__ = ...
generic_edge_match.__doc__ = ...



from typing import Callable, Sequence
from .patches import Rectangle
from .offsetbox import OffsetBox
from .lines import Line2D
from .legend import Legend
from .container import BarContainer
from .artist import Artist

from collections.abc import Sequence

def update_from_first_child(tgt: Rectangle, src: BarContainer) -> None: ...

class HandlerBase:
    def __init__(
        self,
        xpad: float = ...,
        ypad: float = ...,
        update_func: Callable = ...,
    ) -> None: ...
    def update_prop(
        self, legend_handle: Artist, orig_handle: Artist, legend: Legend
    ): ...
    def adjust_drawing_area(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
    ): ...
    def legend_artist(
        self, legend: Legend, orig_handle: Artist, fontsize: int, handlebox: OffsetBox
    ): ...
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerNpoints(HandlerBase):
    def __init__(
        self, marker_pad: float = ..., numpoints: None = ..., **kwargs
    ) -> None: ...
    def get_numpoints(self, legend: Legend): ...
    def get_xdata(
        self, legend: Legend, xdescent, ydescent, width, height, fontsize: int
    ): ...

class HandlerNpointsYoffsets(HandlerNpoints):
    def __init__(
        self, numpoints: int = ..., yoffsets: Sequence[float] = ..., **kwargs
    ) -> None: ...
    def get_ydata(
        self, legend: Legend, xdescent, ydescent, width, height, fontsize: int
    ): ...

class HandlerLine2DCompound(HandlerNpoints):
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class _Line2DHandleList(Sequence):
    def __init__(self, legline: Line2D) -> None: ...
    def __len__(self): ...
    def __getitem__(self, index: int) -> Line2D: ...

class HandlerLine2D(HandlerNpoints):
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerPatch(HandlerBase):
    def __init__(self, patch_func: None = ..., **kwargs) -> None: ...
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerStepPatch(HandlerBase):
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerLineCollection(HandlerLine2D):
    def get_numpoints(self, legend: Legend): ...
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerRegularPolyCollection(HandlerNpointsYoffsets):
    def __init__(self, yoffsets=..., sizes=..., **kwargs) -> None: ...
    def get_numpoints(self, legend: Legend): ...
    def get_sizes(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
    ): ...
    def update_prop(
        self, legend_handle: Artist, orig_handle: Artist, legend: Legend
    ): ...
    def create_collection(self, orig_handle, sizes, offsets, offset_transform): ...
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerPathCollection(HandlerRegularPolyCollection):
    def create_collection(
        self, orig_handle: Artist, sizes, offsets, offset_transform
    ): ...

class HandlerCircleCollection(HandlerRegularPolyCollection):
    def create_collection(
        self, orig_handle: Artist, sizes, offsets, offset_transform
    ): ...

class HandlerErrorbar(HandlerLine2D):
    def __init__(
        self,
        xerr_size: float = ...,
        yerr_size: float = ...,
        marker_pad: float = ...,
        numpoints: int = ...,
        **kwargs
    ) -> None: ...
    def get_err_size(
        self, legend: Legend, xdescent, ydescent, width, height, fontsize: int
    ): ...
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerStem(HandlerNpointsYoffsets):
    def __init__(
        self,
        marker_pad: float = 0.3,
        numpoints: int = ...,
        bottom: float = ...,
        yoffsets: Sequence[float] = ...,
        **kwargs
    ) -> None: ...
    def get_ydata(
        self, legend: Legend, xdescent, ydescent, width, height, fontsize: int
    ): ...
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerTuple(HandlerBase):
    def __init__(self, ndivide: int = 1, pad: float = ..., **kwargs) -> None: ...
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...

class HandlerPolyCollection(HandlerBase):
    def create_artists(
        self,
        legend: Legend,
        orig_handle: Artist,
        xdescent,
        ydescent,
        width,
        height,
        fontsize: int,
        trans,
    ): ...
